from sphinx_vb_domain.vb_domain import VBDomain


# def test_handle_signature():
#     vb_function = VBDomain.VBFunction()
#     result = vb_function.handle_signature(
#         "Public Function MyFunction(x As Integer) As String")
#     assert result['access_modifier'] == 'Public'
#     assert result['func_type'] == 'Function'
#     assert result['func_name'] == 'MyFunction'
#     assert result['args'] == 'x As Integer'
#     assert result['return_type'] == 'String'


def test_dummy():
    assert True
